import 'package:flutter/material.dart';

enum Language {
  id,
  en,
}

Map<String, Map<Language, String>> _translations = {};
Language _currentLocale = Language.en;

resetTranslation() {
  _translations = {};
}

setTranslation(Map<String, Map<Language, String>> translations) {
  for (var key in translations.keys) {
    if (_translations.containsKey(key)) {
      _translations[key].addAll(translations[key]);
    } else {
      _translations[key] = translations[key];
    }
  }
}

extension Translation on String {
  String get tr {
    final key = this;

    if (_translations.containsKey(key)) {
      final trs = _translations[key];
      if (trs.containsKey(_currentLocale)) {
        return trs[_currentLocale];
      }

      if (trs.isNotEmpty) {
        return trs.values.first;
      }
    }

    return key;
  }
}

class LocalizationDelegate extends LocalizationsDelegate<_MyLocalization> {
  static LocalizationDelegate delegate = LocalizationDelegate();

  @override
  bool isSupported(Locale locale) => ["en", "id"].contains(locale.languageCode);

  @override
  Future<_MyLocalization> load(Locale locale) async => _MyLocalization(locale);

  @override
  bool shouldReload(covariant LocalizationsDelegate<_MyLocalization> old) =>
      true;
}

class _MyLocalization {
  _MyLocalization(Locale locale) {
    _currentLocale = locale.languageCode == "id" ? Language.id : Language.en;
  }
}
