import 'package:flutter/material.dart';
import 'package:provider/provider.dart';
import 'package:retgoo_sdk/pages/process_activity_list.dart';
import 'package:retgoo_sdk/providers/process_provider.dart';
import 'package:retgoo_sdk/widgets/bpmn_diagram.dart';
import 'package:retgoo_sdk/retgoo_sdk.dart';

class DiagramPage extends StatelessWidget {
  const DiagramPage({Key key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    bool isLarge = MediaQuery.of(context).size.width > 600;
    return Scaffold(
      appBar: AppBar(
        title: Text("Diagram"),
      ),
      body: isLarge
          ? Row(
              crossAxisAlignment: CrossAxisAlignment.stretch,
              children: [
                Expanded(
                  child: ClipRect(child: buildDiagram()),
                ),
                VerticalDivider(
                  width: 1.0,
                ),
                SizedBox(
                  width: 350,
                  child: buildActivityList(context: context),
                )
              ],
            )
          : Stack(
              clipBehavior: Clip.antiAlias,
              children: [
                Positioned.fill(
                  child: buildDiagram(),
                ),
                DraggableScrollableSheet(
                  minChildSize: 0.4,
                  maxChildSize: 1.0,
                  initialChildSize: 0.4,
                  builder: (context, controller) => Material(
                    color: Colors.white,
                    type: MaterialType.canvas,
                    child: buildActivityList(
                      context: context,
                      controller: controller,
                    ),
                  ),
                )
              ],
            ),
    );
  }

  Widget buildActivityList(
      {BuildContext context, ScrollController controller}) {
    return Column(
      crossAxisAlignment: CrossAxisAlignment.stretch,
      children: [
        Padding(
          padding: const EdgeInsets.all(16.0),
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.stretch,
            children: [
              Text(
                "lbl.activities".tr,
                style: TextStyle(
                  fontSize: 24.0,
                  fontWeight: FontWeight.w900,
                ),
              ),
              Text("lbl.activities_description".tr),
            ],
          ),
        ),
        Divider(),
        Expanded(
          child: RefreshIndicator(
            onRefresh: () async {
              await Provider.of<ProcessProvider>(context, listen: false)
                  .refreshToken();

              await Provider.of<ProcessProvider>(context, listen: false)
                  .refreshActivities();
            },
            child: SingleChildScrollView(
              controller: controller,
              child: ProcessActivityList(),
            ),
          ),
        ),
      ],
    );
  }

  Widget buildDiagram() {
    return Consumer<ProcessProvider>(
      builder: (context, provider, child) {
        return BpmnDiagram(
          directForm: provider.directForm,
          tokens: provider.tokens,
        );
      },
    );
  }
}
