import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:provider/provider.dart';
import 'package:retgoo_sdk/forms/forms.dart';
import 'package:retgoo_sdk/models/directform.dart';
import 'package:retgoo_sdk/pages/completed_page.dart';
import 'package:retgoo_sdk/pages/diagram_page.dart';
import 'package:retgoo_sdk/pages/loading_page.dart';
import 'package:retgoo_sdk/pages/not_implemented_page.dart';
import 'package:retgoo_sdk/pages/process_information_page.dart';
import 'package:retgoo_sdk/providers/process_provider.dart';
import 'package:retgoo_sdk/widgets/retgoo_server.dart';
import 'package:shrink_sidemenu/shrink_sidemenu.dart';
import 'package:retgoo_sdk/retgoo_sdk.dart';

class ProcessPage extends StatefulWidget {
  final int idProcessInstance;
  final int idActivityNotification;

  ProcessPage({Key key, this.idActivityNotification, this.idProcessInstance})
      : super(key: key);

  @override
  _ProcessPageState createState() => _ProcessPageState();
}

class _ProcessPageState extends State<ProcessPage> {
  ProcessProvider provider;

  @override
  void initState() {
    super.initState();
    provider = ProcessProvider(
      idProcessInstance: widget.idProcessInstance,
      idNotificationActivity: widget.idActivityNotification,
    );
  }

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    RetGooServer.of(context).setListener("update_client_token", (value) {
      provider.refreshToken();
    });

    RetGooServer.of(context).setListener("task_token_completed", (value) {
      provider.refreshToken();
    });

    RetGooServer.of(context).setListener("activity_token_list", (value) {
      provider.refreshActivities();
    });
  }

  @override
  deactivate() {
    RetGooServer.of(context).removeListener("task_token_completed");
    RetGooServer.of(context).removeListener("activity_token_list");
    RetGooServer.of(context).removeListener("update_client_token");
    super.deactivate();
  }

  Widget build(BuildContext context) {
    return ChangeNotifierProvider.value(
      value: provider,
      child: Scaffold(
        body: ProcessContent(),
      ),
    );
  }
}

class ProcessContent extends StatefulWidget {
  const ProcessContent({
    Key key,
  }) : super(key: key);

  @override
  _ProcessContentState createState() => _ProcessContentState();
}

class _ProcessContentState extends State<ProcessContent> {
  DirectForm directForm;
  Offset offset;

  final menuKey = GlobalKey<SideMenuState>();
  int _currentIndex = 0;
  double _sideButtonSize = 72;

  buildBody() {
    return AnnotatedRegion(
      value: SystemUiOverlayStyle.dark.copyWith(
        statusBarColor: Colors.transparent,
      ),
      child: SideMenu(
        key: menuKey,
        background: Color(0xFFe5e5e5),
        radius: BorderRadius.circular(48.0),
        type: SideMenuType.slideNRotate,
        closeIcon: null,
        menu: buildMenu(),
        child: IgnorePointer(
          ignoring: menuKey.currentState?.isOpened ?? false,
          child: WillPopScope(
            onWillPop: () async {
              if (menuKey.currentState.isOpened) {
                return true;
              }

              menuKey.currentState.openSideMenu();
              return false;
            },
            child: Stack(
              children: [
                Positioned.fill(
                  child: IndexedStack(
                    index: _currentIndex,
                    children: [
                      buildForm(),
                      ProcessInformationPage(),
                      DiagramPage(),
                    ],
                  ),
                ),
                Positioned(
                  top: offset?.dy ??
                      (MediaQuery.of(context).size.height - _sideButtonSize) /
                          2,
                  right: 0,
                  child: _SideButton(
                    size: _sideButtonSize,
                    onMoved: (offset) {
                      setState(() {
                        this.offset = offset;
                      });
                    },
                    onTap: () {
                      menuKey.currentState.openSideMenu();
                    },
                  ),
                ),
              ],
            ),
          ),
        ),
      ),
    );
  }

  buildMenu() {
    return DefaultTextStyle(
      style: DefaultTextStyle.of(context).style.copyWith(
            color: Color(0xFF242424),
          ),
      child: SafeArea(
        top: false,
        child: Padding(
          padding: const EdgeInsets.symmetric(horizontal: 8.0),
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              Padding(
                padding: const EdgeInsets.symmetric(horizontal: 16.0),
                child: Text(
                  directForm.masterProcessName,
                  style: TextStyle(
                    fontSize: 24.0,
                    fontWeight: FontWeight.w900,
                  ),
                ),
              ),
              Divider(),
              buildMenuButton(
                selected: _currentIndex == 0,
                label: "menu.form".tr,
                icon: CupertinoIcons.doc_on_clipboard,
                onTap: () {
                  setState(() {
                    _currentIndex = 0;
                  });
                  menuKey.currentState.closeSideMenu();
                },
              ),
              buildMenuButton(
                selected: _currentIndex == 1,
                label: "menu.information".tr,
                icon: CupertinoIcons.info,
                onTap: () {
                  menuKey.currentState.closeSideMenu();
                  setState(() {
                    _currentIndex = 1;
                  });
                },
              ),
              buildMenuButton(
                selected: _currentIndex == 2,
                label: "menu.diagram".tr,
                icon: CupertinoIcons.flowchart,
                onTap: () {
                  setState(() {
                    _currentIndex = 2;
                  });
                  menuKey.currentState.closeSideMenu();
                },
              ),
              Divider(),
              ListTile(
                title: Text("menu.close".tr),
                leading: Icon(CupertinoIcons.chevron_back),
                onTap: () {
                  Navigator.pop(context);
                },
              ),
            ],
          ),
        ),
      ),
    );
  }

  buildMenuButton({
    String label,
    IconData icon,
    bool selected,
    GestureTapCallback onTap,
  }) {
    return Material(
      borderRadius: BorderRadius.circular(100.0),
      clipBehavior: Clip.antiAliasWithSaveLayer,
      color: selected ? Colors.black.withOpacity(0.125) : Colors.transparent,
      textStyle: TextStyle(
        color: Color(0xFF181818),
      ),
      child: InkWell(
        onTap: onTap,
        child: Padding(
          padding: const EdgeInsets.all(16.0),
          child: Row(
            children: [
              Icon(
                icon,
                color: Color(0xFF181818),
              ),
              const SizedBox(
                width: 16.0,
              ),
              Text(
                label,
                style: TextStyle(
                  fontSize: 16.0,
                ),
              ),
            ],
          ),
        ),
      ),
    );
  }

  var userTask;
  buildForm() {
    return Selector<ProcessProvider, ProcessState>(
      selector: (context, provider) => provider.state,
      builder: (context, state, child) {
        if (state == ProcessState.completed) {
          return CompletedPage();
        }

        final directForm =
            Provider.of<ProcessProvider>(context, listen: false).directForm;
        final registrationId = directForm.encodeTaskId;

        if (!registeredUserTasks.containsKey(registrationId)) {
          return NotImplementedPage(directForm: directForm);
        }

        if (userTask == null) {
          userTask = registeredUserTasks[registrationId]();
        }

        return userTask.attach(context);
      },
    );
  }

  @override
  Widget build(BuildContext context) {
    return Selector<ProcessProvider, DirectForm>(
      selector: (context, provider) => provider.directForm,
      builder: (context, directForm, child) {
        if (directForm == null) {
          return LoadingPage();
        }

        this.directForm = directForm;
        return Scaffold(
          body: buildBody(),
        );
      },
    );
  }
}

class _SideButton extends StatefulWidget {
  final double size;
  final ValueChanged<Offset> onMoved;
  final GestureTapCallback onTap;
  const _SideButton({
    Key key,
    this.size = 64,
    this.onMoved,
    this.onTap,
  }) : super(key: key);

  @override
  _SideButtonState createState() => _SideButtonState();
}

class _SideButtonState extends State<_SideButton>
    with TickerProviderStateMixin {
  final buttonKey = GlobalKey();

  @override
  Widget build(BuildContext context) {
    return AnimatedOpacity(
      opacity: 0.5,
      duration: Duration(milliseconds: 200),
      child: Draggable(
        affinity: Axis.vertical,
        axis: Axis.vertical,
        childWhenDragging: Container(),
        onDragEnd: (details) {
          double y = details.offset.dy;
          if (y < 0) {
            y = 0;
          }

          if (y > MediaQuery.of(context).size.height - widget.size) {
            y = MediaQuery.of(context).size.height - widget.size;
          }

          final offset = Offset(details.offset.dx, y);
          if (widget.onMoved != null) {
            widget.onMoved(offset);
          }
        },
        feedback: buildSideMenu(),
        child: GestureDetector(
          onTap: widget.onTap,
          child: buildSideMenu(),
        ),
      ),
    );
  }

  Widget buildSideMenu() {
    return Container(
      key: buttonKey,
      height: widget.size,
      width: widget.size / 2,
      decoration: BoxDecoration(
        color: Colors.black.withOpacity(0.5),
        borderRadius: BorderRadius.only(
          topLeft: Radius.circular(widget.size * 2),
          bottomLeft: Radius.circular(widget.size * 2),
        ),
      ),
      padding: const EdgeInsets.all(4.0),
      child: Padding(
        padding: const EdgeInsets.all(8.0),
        child: Icon(
          CupertinoIcons.fullscreen_exit,
          color: Colors.white,
        ),
      ),
    );
  }
}
