import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:firebase_auth/firebase_auth.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';
import 'package:retgoo_sdk/models/user.dart';
import 'package:retgoo_sdk/pages/user_login_page.dart';
import 'package:retgoo_sdk/providers/authentication_provider.dart';
import 'package:retgoo_sdk/retgoo_sdk.dart';

class UserLoginDashboard extends StatelessWidget {
  const UserLoginDashboard({Key key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        leading: Padding(
          padding: const EdgeInsets.all(8.0),
          child: CircleAvatar(
            backgroundImage:
                NetworkImage(FirebaseAuth.instance.currentUser.photoURL),
          ),
        ),
        title: Text("lbl.select_user".tr),
        actions: [
          IconButton(
            icon: Icon(CupertinoIcons.option),
            onPressed: () {
              Provider.of<AuthenticationProvider>(context, listen: false)
                  .logout();
            },
          )
        ],
      ),
      body: buildBody(),
    );
  }

  buildBody() {
    return StreamBuilder<QuerySnapshot>(
      stream: FirebaseFirestore.instance
          .collection("users/${FirebaseAuth.instance.currentUser.uid}/portals")
          .snapshots(),
      builder: (context, snapshot) {
        if (snapshot.connectionState == ConnectionState.waiting) {
          return Center(
            child: CircularProgressIndicator(),
          );
        }

        final items = snapshot.data.docs;
        return ListView.builder(
          itemCount: items.length + 1,
          itemBuilder: (context, index) {
            if (index < items.length) {
              final item = items.elementAt(index);
              return ListTile(
                leading: Icon(CupertinoIcons.person_circle),
                trailing: IconButton(
                  icon: Icon(CupertinoIcons.delete),
                  onPressed: () {
                    item.reference.delete();
                  },
                ),
                title: Text(item["nick"]),
                subtitle: Text("${item["username"]}@${item["host"]}"),
                onTap: () {
                  Provider.of<AuthenticationProvider>(context, listen: false)
                      .setUser(
                          UserAccount()..accessToken = item["access_token"]);
                },
              );
            }

            return ListTile(
              leading: Icon(CupertinoIcons.add),
              title: Text("Add New Account"),
              subtitle: Text("Sign into another account"),
              onTap: () {
                Navigator.push(
                  context,
                  MaterialPageRoute(
                    builder: (context) => LoginPage(),
                  ),
                );
              },
            );
          },
        );
      },
    );
  }
}
