import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:provider/provider.dart';
import 'package:retgoo_sdk/providers/authentication_provider.dart';
import 'package:retgoo_sdk/lang/translation.dart';

class LoginPage extends StatefulWidget {
  LoginPage({Key key}) : super(key: key);

  @override
  _LoginPageState createState() => _LoginPageState();
}

class _LoginPageState extends State<LoginPage> {
  final txtUsername = TextEditingController();
  final txtPassword = TextEditingController();

  @override
  void dispose() {
    txtUsername.dispose();
    txtPassword.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: buildBody(),
    );
  }

  buildBody() {
    return AnnotatedRegion(
      value: SystemUiOverlayStyle.light.copyWith(
        statusBarColor: Colors.transparent,
      ),
      child: Container(
        decoration: BoxDecoration(
          gradient: RadialGradient(
            colors: [
              Color(0xFFe5e5e5),
              Color(0xFFf8f8f8),
            ],
          ),
        ),
        child: CustomPaint(
          painter: _Painter(
            color: Theme.of(context).primaryColor,
          ),
          child: Center(
            child: Card(
              child: SizedBox(
                width: 350,
                child: Padding(
                  padding: const EdgeInsets.all(24.0),
                  child: Column(
                    mainAxisSize: MainAxisSize.min,
                    mainAxisAlignment: MainAxisAlignment.center,
                    crossAxisAlignment: CrossAxisAlignment.stretch,
                    children: [
                      Text(
                        "lbl.login".tr,
                        style: TextStyle(
                          fontSize: 32.0,
                          fontWeight: FontWeight.w900,
                        ),
                      ),
                      Text("lbl.login_description".tr),
                      const SizedBox(
                        height: 16.0,
                      ),
                      TextField(
                        controller: txtUsername,
                        decoration: InputDecoration(
                          border: OutlineInputBorder(
                            borderRadius: BorderRadius.circular(100.0),
                            borderSide: BorderSide(
                              color: Theme.of(context).dividerColor,
                            ),
                          ),
                          hintText: "lbl.username".tr,
                          prefixIcon: Icon(CupertinoIcons.person_circle),
                        ),
                        autofillHints: [
                          AutofillHints.username,
                        ],
                      ),
                      const SizedBox(
                        height: 8.0,
                      ),
                      TextField(
                        obscureText: true,
                        controller: txtPassword,
                        decoration: InputDecoration(
                          border: OutlineInputBorder(
                            borderRadius: BorderRadius.circular(100.0),
                            borderSide: BorderSide(
                              color: Theme.of(context).dividerColor,
                            ),
                          ),
                          hintText: "lbl.password".tr,
                          prefixIcon: Icon(CupertinoIcons.lock_circle),
                        ),
                        autofillHints: [
                          AutofillHints.password,
                        ],
                      ),
                      const SizedBox(
                        height: 8.0,
                      ),
                      FlatButton.icon(
                        icon: Icon(CupertinoIcons.arrow_right_circle),
                        label: Text("btn.login".tr),
                        padding: const EdgeInsets.all(16.0),
                        onPressed: () async {
                          final result =
                              await Provider.of<AuthenticationProvider>(context,
                                      listen: false)
                                  .login(
                            username: txtUsername.text,
                            password: txtPassword.text,
                          );
                          if (result != null) {
                            Navigator.pop(context);
                          }
                        },
                      )
                    ],
                  ),
                ),
              ),
            ),
          ),
        ),
      ),
    );
  }
}

class _Painter extends CustomPainter {
  final Color color;
  _Painter({this.color = Colors.indigo});

  @override
  void paint(Canvas canvas, Size size) {
    Path p = Path()
      ..moveTo(0, 0)
      ..lineTo(0, size.height / 2)
      ..quadraticBezierTo(size.width / 2,
          size.height / 2 + (size.height * 0.05), size.width, size.height / 2)
      ..lineTo(size.width, 0.0)
      ..close();

    canvas.drawPath(
      p,
      Paint()
        ..style = PaintingStyle.fill
        ..color = color,
    );
  }

  @override
  bool shouldRepaint(covariant CustomPainter oldDelegate) => false;
}
